/* *******************************************************************
   Webserver - the basic functions
   ***************************************************************** */

void checkForClient()
{
  EthernetClient client = server.available();

  if (client) {
    Serial.println(F("\n[server] client connected"));
    uint8_t i = 0;                                // index / current read position
    const uint16_t buffersize = 100;              // size of read buffer (reads a complete line) (if larger than 255, modify i also!
    const uint16_t smallbuffersize = 30;          // a smaller buffer for results
    char lineBuffer[buffersize] {'\0'};           // buffer for incomming data
    char method[8];                               // largest one 7+1. HTTP request methods in RFC7231 + RFC5789: GET HEAD POST PUT DELETE CONNECT OPTONS TRACE PATCH
    char uri[smallbuffersize];                    // the requestet page, shorter than smallbuffersize - method
    char requestParameter[smallbuffersize];       // parameter appended to the URI after a ?
    char postParameter[smallbuffersize];          // parameter transmitted in the body / by POST
    enum class Status {REQUEST, CONTENT_LENGTH, EMPTY_LINE, BODY};
    Status status = Status::REQUEST;

    const size_t MESSAGE_BUFFER_SIZE = 64;                  // size of buffer for StreamLib
    char buffer[MESSAGE_BUFFER_SIZE];                       // a buffer for the StreamLib
    BufferedPrint message(client, buffer, sizeof(buffer));  // the StreamLib object to replace client print

    while (client.connected()) {
      while (client.available()) {
        char c = client.read();
        Serial.print(c);  // Debug print received characters to Serial monitor
        if ( c == '\n' )
        {
          if (status == Status::REQUEST)               // read the first line
          {
            //Serial.print(F("lineBuffer="));Serial.println(lineBuffer);
            // now split the input
            char *ptr;
            ptr = strtok(lineBuffer, " ");  // strtok willdestroy the newRequest
            strlcpy(method, ptr, smallbuffersize);
            Serial.print(F("method=")); Serial.println(method);
            ptr = strtok(NULL, " ");
            strlcpy(uri, ptr, smallbuffersize);   // enthält noch evtl. parameter
            if (strchr(uri, '?') != NULL)
            {
              ptr = strtok(uri, "?");  // split URI from parameters
              strcpy(uri, ptr);
              ptr = strtok(NULL, " ");
              strcpy(requestParameter, ptr);
              Serial.print(F("requestParameter=")); Serial.println(requestParameter);
            }
            Serial.print(F("uri=")); Serial.println(uri);
            status = Status::EMPTY_LINE;                          // jump to next status
          }
          else if (status == Status::CONTENT_LENGTH)              // MISSING check for Content-Length
          {
            status = Status::EMPTY_LINE;
          }
          else if (status > Status::REQUEST && i < 2)            // check if we have an empty line
          {
            status = Status::BODY;
          }
          else if (status == Status::BODY)
          {
            strlcpy(postParameter, lineBuffer, smallbuffersize);
            break; // we have received one line payload and break out
          }
          i = 0;
          strcpy(lineBuffer, "");
        }
        else
        {
          if (i < buffersize)
          {
            lineBuffer[i] = c;
            i++;
            lineBuffer[i] = '\0';
          }
          // MISSING wenn status 3 und content-length --> abbrechen.
        }
      }
      if (status == Status::BODY) // status 3 could end without linefeed, therefore we takeover here also
      {
        strlcpy(postParameter, lineBuffer, smallbuffersize);
      }
      Serial.print(F("postParameter=")); Serial.println(postParameter);
      // Simple evaluation of postParameter from body
      // Post data looks like pinD2=On
      if ( strncmp( postParameter, "pinD", 4) == 0 ) {
        byte pin = postParameter[4] - 48; // Convert ascii to int
        // Serial.println(pin);
        if ( strncmp( postParameter + 5, "=On", 3) == 0 ) {
          digitalWrite(pin, 1);
        }
        else if ( strncmp( postParameter + 5, "=Off", 4) == 0 ) {
          digitalWrite(pin, 0);
        }
      }

      // send back a response
      if (!strcmp(uri, "/"))                         // the homepage
        sendPage(message);
      else if (!strcmp(uri, "/1.htm"))
        sendPage1(message);
      else if (!strcmp(uri, "/1.js")) 
        sendJs(message);                                   // javascript calling JSON request
      else if (!strcmp(uri, "/1.json"))                    // send data as json
        sendJson(message);
      else if (!strcmp(uri, "/favicon.ico"))               // a favicon
        //sendFavicon(client);
        send204(message);                                // if you don't have a favicon, send a defined empty response
      else                                                 // if the page is unknown, send HTTP response code 404
        send404(message);
      message.flush();
      client.stop();
    }
  }
}


void sendPage(BufferedPrint &message)
{
  // Serial.println("[server] 200 response send");
  message.println(F("HTTP/1.0 200 OK\r\n"                 // \r\n header fields are terminated by a carriage return (CR) and line feed (LF) character sequence.
                    "Content-Type: text/html\r\n"         // The media type of the body of the request (used with POST and PUT requests).
                    "\r\n"                                // a blank line to split HTTP header and HTTP body
                    "<!doctype html>\n"                   // the HTTP page itself
                    "<html lang='en'>\n"
                    "<head>\n"
                    "<meta charset='utf-8'>\n"
                    "<meta name='viewport' content='width=device-width'>\n"
                    "<title>Webserver as pin controller</title>\n"
                    "</head>\n"
                    "<body style='font-family:Helvetica, sans-serif'>\n"  // a minimum style to avoid serifs
                    "<h1>Webserver as pin controller</h1>\n"
                    "<p>Buttons turn pins on or off</p>\n"
                    "<form method='post' action='/' name='pins'>"));
  for (auto &pin : ledPin)
  {
    //client.print(F("<p>"));
    message.print(pin);
    message.print(F(" <input name='pinD"));
    message.print(pin);
    message.print(F("' type='submit' value='On'>"));
    message.print(F("<input name='pinD"));
    message.print(pin);
    message.print(F("' type='submit' value='Off'>"));
    if (digitalRead(pin)) message.print(F(" active"));
    message.print(F( "<br>\n"));
  }
  message.print (F("</form>\n"));

  message.print (F("</body>\n</html>"));
  message.flush();
}


void send404(BufferedPrint &message)
{
  // Serial.println("[server] response 404 file not found");
  message.println(F("HTTP/1.0 404 Not Found\r\n"
                    "Content-Type: text/plain\r\n"              // we will send a simple plain text without html tags
                    "\r\n"
                    "File Fot Found"));
  message.flush();
}


void send204(BufferedPrint &message)
{
  // Serial.println("[server] response 204 no content");
  message.println(F("HTTP/1.0 204 no content\r\n"));  // no content
  message.flush();
}

void sendFavicon(EthernetClient &client)
{
  client.println("HTTP/1.0 204 no content\r\n");
}
