/* *******************************************************************
   Webserver - page update with Fetch/API
   ***************************************************************** */

void sendPage1(BufferedPrint &message)
{
  // Serial.println("[server] 200 response send");
  message.print  (F("HTTP/1.0 200 OK\r\n"                 // \r\n header Fields are terminated by a carriage return (CR) and line feed (LF) character sequence.
                    "Content-Type: text/html\r\n"         // The media type of the body of the request (used with POST and PUT requests).
                    "\r\n"                                // a blank line to split HTTP header and HTTP body
                    "<!doctype html>\n"                   // the HTTP page itself
                    "<html lang='en'>\n"
                    "<head>\n"
                    "<meta charset='utf-8'>\n"
                    "<meta name='viewport' content='width=device-width'>\n"
                    "<script src='/1.js'></script>\n"
                    "<title>Webserver as pin controller</title>\n"
                    "</head>\n"
                    "<body style='font-family:Helvetica, sans-serif'>\n"  // a minimum style to avoid serifs
                    "<main>\n"
                    "<h1>Arduino Webserver - Fetch Example</h1>\n"));

  message.print(F("<h2>Uptime</h2>\n"
                  "<p>Webserver running since <span id='ss'>"));
  message.print(millis() / 1000);
  message.print(F("</span> seconds</p>\n"));
  message.print(F("<p><span id='hour'></span> <span id='min'></span> <span id='sec'></span></p>\n"));

  message.print(F("<h2>Analog Pins</h2>\n"));
  message.print(F("<p>"));
  for (size_t i; i < sizeof(analogPin); i++)
  {
    message.print(F("Analog pin "));
    message.print(analogPin[i]);
    message.print(F(" = <span id='analog"));
    message.print(i);
    message.print(F("'>"));
    message.print(analogRead(analogPin[i]));
    message.print(F("</span><br>\n"));
  }
  message.print (F("</p>\n"));

  message.print(F("<h2>Digital Pins</h2>\n"));
  message.print(F("<p>"));
  for (size_t i; i < sizeof(ledPin); i++)
  {
    message.print(F("Digital pin "));
    message.print(ledPin[i]);
    message.print(F(" = <span id='digital"));
    message.print(i);
    message.print(F("'>"));
    message.print(digitalRead(ledPin[i]));
    message.print(F("</span><br>\n"));
  }
  message.print (F("</p>\n"));
  
  message.print (F("</main>\n"
                   "</body>\n"
                   "</html>"));
  message.flush();
}


void sendJs(BufferedPrint &message)
{
  message.print(F("HTTP/1.0 200\r\n"
                  "Content-Type: application/javascript\r\n"
                  "Cache-Control: public, max-age=2678400\r\n"
                  "\r\n"));
  message.print(F("const url ='1.json';\n"
                  "function renew(){\n"
                  " document.getElementById('ss').style.color = 'blue'\n"
                  " fetch(url)\n" // Call the fetch function passing the url of the API as a parameter
                  " .then(response => {return response.json();})\n"
                  " .then(jo => {\n"
                  "   document.getElementById('sec').innerHTML = Math.floor(jo['ss'] % 60); \n"
                  "   for (var i in jo)\n"
                  "    {if (document.getElementById(i)) document.getElementById(i).innerHTML = jo[i];}\n"
                  // add other fields here (e.g. the delivered JSON name doesn't fit to the html id
                  // finally, update the runtime
                  "   if (jo['ss'] > 60) { document.getElementById('min').innerHTML = Math.floor(jo['ss'] / 60 % 60);}\n"
                  "   if (jo['ss'] > 3600) {document.getElementById('hour').innerHTML = Math.floor(jo['ss'] / 3600 % 24);}\n"
                  "   if (jo['ss'] > 86400) {document.getElementById('day').innerHTML = Math.floor(jo['ss'] / 86400 % 7);}\n"
                  "   if (jo['ss'] > 604800) {document.getElementById('week').innerHTML = Math.floor(jo['ss'] / 604800 % 52);}\n"
                  "   document.getElementById('ss').style.color = 'black';\n"
                  " })\n"
                  " .catch(function() {\n"
                  "  document.getElementById('ss').style.color = 'red';\n"
                  " });\n"
                  "}\n"
                  "document.addEventListener('DOMContentLoaded', renew, setInterval(renew, "));
  message.print(F("5000"));  // the update interval in milliseconds
  message.print(F("));"));
  message.flush();
}

void sendJson(BufferedPrint &message)
//void getJson(EthernetClient &client)
// send the data as JSON to the web browser
{
  message.print(F("HTTP/1.0 200\r\n"
                  "Content-Type: application/json\r\n"
                  "\r\n"
                  "{\"ss\":"));
  message.print(millis() / 1000);                              // timestamp in seconds

  for (size_t i = 0; i < sizeof(ledPin); i++)
  {
    message.print(F(",\"digital"));
    message.print(i);
    message.print(F("\":"));
    message.print(digitalRead(ledPin[i]));
  }

  for (size_t i = 0; i < sizeof(analogPin); i++)
  {
    message.print(F(",\"analog"));
    message.print(i);
    message.print(F("\":"));
    message.print(analogRead(analogPin[i]));
  }
  message.println(F("}"));
  message.flush();
}
